//+------------------------------------------------------------------+
//|                                                 PM_Vachash_7.mq4 |
//|                                             Copyright  2012, PM |
//|                                                   mql@hotmail.nl |
//|                                                     Version 0.03 |
//|                                   Last Updated November 20, 2012 |
//+------------------------------------------------------------------+
//                          Donations to:                           //
//                                                                  //
//                      PayPal: mql@hotmail.nl                      //
//+------------------------------------------------------------------+
#property copyright "Copyright  2012, PM"
#property link      "mql@hotmail.nl"

//+------------------------------------------------------------------+
//| External paramaters which can be set by the user.                |
//+------------------------------------------------------------------+
extern double LotSize               = 0.01;     // Lotsize to be used for opening a new order.
extern int    MagicNumber           = 12110811; // Unique identifier to group orders together
extern int    TrailingSLValue       = 0;       // Trailing Stop Value. When this value is 0 it is disabled.

extern int    TrendLordIndicatorPeriod = 50;    // Indicator Period used for the TrendLord indicator

extern bool   CloseOrdersOnFriday   = True;     // Do we want to close all open orders before the weekend, every friday?
extern int    CloseOrdersHourTime   = 16;       // What hour do we want the orders to be closed?

extern string TrendLordIndieName    = "PM_TrendLord"; // TrendLord


//+------------------------------------------------------------------+
//| Internal used variables.                                         |
//+------------------------------------------------------------------+
string _tag = "PM_JF_TRENDLORD_EA";             // Used for logging issues
string _FriendlyName = "TrendLord EA";      // Name which will be used to display errors and warnings.
string _symbol;                             // Symbol where the ea is attached to
int    _digits, _period;                    // Digits and the current period
double _point;                              // Point being used for this pair
static datetime _lastBarTime = 0;           // DateTime of the last new bar which opened
double          _orderInfo[20];             // Contains information about the opened orders.

datetime _lastOpenedTrade;


int   _signal[1];                           // Contains the current and the previous signal of the TrendLord indicator

double _tlValues[6];
bool   _CanOpenNewOrder;

/*
Hello Phil,
did you had time to take a look at the Indicator?
So, about the E/A.... see attached Picture, in the upper
you can see the Renko Bars, in the lower you can see the
Indicator.
How shall the E/A work...
 
If the Indicator displays a uptrend (blue) we go long,
we close the long position and open in the same time a
short position if the Indicator displays a downtrend (red),
the short position is valid as long we have the down trend
(red), if the Indicator shows again a uptrend (blue) we close the
short position and open in the same time a short position.... and so
on.. :-)
 
The E/A should run 24 hours on a server, therefore we need to develope
a protection, a save restart routine. Possible?
 
*/

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
    // Store most common used pointers into variables for better performance.
    _period       = Period() ;   
    _symbol       = Symbol() ;
    _digits       = Digits ;   
    _point        = Point ; 
    
    // store bartime so upon initializing no order will be opened. By desgin.    
//    _lastBarTime = iTime(_symbol,_period,0); 
    _lastBarTime = Time[0];  
    _signal[0]=-1;
    _signal[1]=-1;
    
    
    _tlValues[0]=0;
    _tlValues[1]=0;
    _tlValues[2]=0;
    _tlValues[3]=0;
    _tlValues[4]=0;
    _tlValues[5]=0;
    
    string name = TrendLordIndieName;
    
    _tlValues[3]= iCustom(NULL,0,name,TrendLordIndicatorPeriod,0,2);
    _tlValues[4]= iCustom(NULL,0,name,TrendLordIndicatorPeriod,1,2);
    _tlValues[5]= iCustom(NULL,0,name,TrendLordIndicatorPeriod,2,2);
    
    CollectData();

    _CanOpenNewOrder = true;
    
    return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
    ObDeleteObjectsByPrefix(_tag);
    Comment(""); 

    return(0);
  }
  
  
  
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
    
    // Check whether the flag for opening orders can be resetted...
    if(!_CanOpenNewOrder)   {
        if(!TimeDayOfWeek(TimeCurrent()==5)) _CanOpenNewOrder=true;
        else if((TimeDayOfWeek(TimeCurrent()==5)) && (TimeHour(TimeCurrent()<CloseOrdersHourTime))) _CanOpenNewOrder=true;
    }
    
    //-----------------------------------------------------
    // Only process certain tasks when a new bar has begun.
    //-----------------------------------------------------
    if(NewBar()==true)  {   
        // collect all neccesarry data which will be used later on.
        CollectData();

        // Processes the current data, tick by tick.
        if(_CanOpenNewOrder==true) ProcessData();
    }  
    
    // Close all orders on friday?
    if(CloseOrdersOnFriday==true)   {
        if((TimeDayOfWeek(TimeCurrent())==5) && (TimeHour(TimeCurrent())==CloseOrdersHourTime)) {
            CloseOrders(0);
            CloseOrders(1);
            _CanOpenNewOrder=false;
        }
    }    

    // Show order status and values.
    DisplayCommentSection();
    

    
    //TODO: More Logic here...

    return(0);
  }
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
//| Collects all required data at each new bar open.                 |
//| Gathered data will be used to determine new opening of orders.   |
//+------------------------------------------------------------------+
void CollectData()
{   
    // TODO: implement array to be filled with data

    // store previously determined trend signal
    _signal[0]=_signal[1];
    

    _tlValues[0]=_tlValues[3];
    _tlValues[1]=_tlValues[4];
    _tlValues[2]=_tlValues[5];
    
    string name = TrendLordIndieName;
    

    double trendLordValue0=0, trendLordValue1=0, trendLordValue2 = 0;    

    // Determine trend of last closed bar
    trendLordValue0 = iCustom(NULL,0,name,TrendLordIndicatorPeriod,0,1);
    trendLordValue1 = iCustom(NULL,0,name,TrendLordIndicatorPeriod,1,1);
    trendLordValue2 = iCustom(NULL,0,name,TrendLordIndicatorPeriod,2,1);

    _tlValues[3]=trendLordValue0 ;
    _tlValues[4]=trendLordValue1;
    _tlValues[5]=trendLordValue2;
    
    if((trendLordValue1==EMPTY_VALUE) && (!trendLordValue2==EMPTY_VALUE)) _signal[1]=1;
    else if((trendLordValue2==EMPTY_VALUE) && (!trendLordValue1==EMPTY_VALUE)) _signal[1]=0;
    else _signal[1]=-1;

}

//+---------------------------------------------------------------------+
//| Processes the current tickdata and controls the trades accordingly. |
//+---------------------------------------------------------------------+
void ProcessData()
{

    int ticketId;
    // Is Bar2 Short?
    if((_tlValues[1]==EMPTY_VALUE) && (!_tlValues[2]==EMPTY_VALUE))  {
        // Is Bar1 Long?
        if((_tlValues[5]==EMPTY_VALUE) && (!_tlValues[4]==EMPTY_VALUE)) {
            // Open new long sinc direction changed from short to long...
            ticketId =  OpenOrder(OP_BUY, LotSize, "Short To Long");
            // Close any opened short orders since direction has changed...
            CloseOrders(1);  
        }
    } 

    // Is Bar2 Long?
    else if((_tlValues[2]==EMPTY_VALUE) && (!_tlValues[1]==EMPTY_VALUE))  {
        // Is Bar1 Short?
        if((_tlValues[4]==EMPTY_VALUE) && (!_tlValues[5]==EMPTY_VALUE)) {
            // Open new short since direction changed from long to short...
            ticketId =  OpenOrder(OP_SELL, LotSize, "Long To Short");
            // Close any opened long order(s) since direction has changed...
            CloseOrders(0);  
       }
    } 

    // When enabled check the trailing stop against all currently opened orders.
    if(!TrailingSLValue==0) CheckTrailingStop();
}

string SignalText(int value)
{
    if(value==0) return("Long");
    if(value==1) return("Short");
    return("N/A");
    
}


//+------------------------------------------------------------------+
//| When running the EA in debugmode the values of the orderArray    |
//| will be displayed in the comment section.                        |
//+------------------------------------------------------------------+
void DisplayCommentSection()
{
    string comment="";
    // basic information regarding time, lotsize and entry and exit values, calculated on new bar.
    comment = "-------------------------------------------------------------" + "\n" +
              "* ServerTime : " + TimeToStr(TimeCurrent(),TIME_DATE|TIME_MINUTES) + "\n" +
              "-------------------------------------------------------------" + "\n";

    comment = comment + "* Current barTime : " + TimeToStr(Time[0],TIME_DATE|TIME_MINUTES) + "\n";
    comment = comment + "* Previous barTime: " + TimeToStr(Time[1],TIME_DATE|TIME_MINUTES) + "\n";

    if(_CanOpenNewOrder)
        comment = comment + "* TRADING ALLOWED." + "\n";
    else
        comment = comment + "* TRADING NOT ALLOWED." + "\n";

    comment = comment + "-------------------------------------------------------------" + "\n";
/**/
    comment = comment + "* Last closed TrendBar  : " + SignalText(_signal[0]) + "\n";


    comment = comment + "* TrendLord1-0 : " + DoubleToStr(_tlValues[3],4) + "\n";
    comment = comment + "* TrendLord1-1 : " + DoubleToStr(_tlValues[4],4) + "\n";
    comment = comment + "* TrendLord1-2 : " + DoubleToStr(_tlValues[5],4) + "\n";
/**/  
    if((_tlValues[4]==EMPTY_VALUE) && (!_tlValues[5]==EMPTY_VALUE)) comment = comment + "* N-1: SIGNAL=RED SHORT" + "\n";
    else if((_tlValues[5]==EMPTY_VALUE) && (!_tlValues[4]==EMPTY_VALUE)) comment = comment + "* N-1: SIGNAL=GREEN LONG" + "\n";
    else comment = comment + "* N-1: SIGNAL=WAIT UNKNOWN" + "\n";

/**/    
    comment = comment + "-------------------------------------------------------------" + "\n";
        
    comment = comment + "* Last closed+1 TrendBar: " + SignalText(_signal[1]) + "\n";
    comment = comment + "* TrendLord0-0 : " + DoubleToStr(_tlValues[0],4) + "\n";
    comment = comment + "* TrendLord0-1 : " + DoubleToStr(_tlValues[1],4) + "\n";
    comment = comment + "* TrendLord0-2 : " + DoubleToStr(_tlValues[2],4) + "\n";
/**/
    if((_tlValues[1]==EMPTY_VALUE) && (!_tlValues[2]==EMPTY_VALUE)) comment = comment + "* N-2: SIGNAL=RED SHORT" + "\n";
    else if((_tlValues[2]==EMPTY_VALUE) && (!_tlValues[1]==EMPTY_VALUE)) comment = comment + "* N-2: SIGNAL=GREEN LONG" + "\n";
    else comment = comment + "* N-2: SIGNAL=WAIT UNKNOWN" + "\n";

    //int   _signal[1];                           // Contains the current and the previous signal of the TrendLord indicator
//    OP_SELL=1, OP_BUY=0, NONE=-1


    // Get information regarding the orders for this pair.
    CheckOrderInformation();
    int openOrders = OrderCount(-1);
    // Display order information
    comment = comment + "-------------------------------------------------------------" + "\n";
    comment = comment + "* Open orders: " + openOrders + "\n";
    
    
    if(!openOrders==0)
        comment = comment + "* Order opened at: " + TimeToStr(_lastOpenedTrade,TIME_DATE|TIME_MINUTES) + "\n";
    
    
    comment = comment + "* Open Buy and Lotsize : " + DoubleToStr(_orderInfo[0],0) + " / " + DoubleToStr(_orderInfo[5],2) + "\n";
    comment = comment + "* Open Sell and Lotsize: " + DoubleToStr(_orderInfo[1],0) + " / " + DoubleToStr(_orderInfo[6],2) + "\n";
    comment = comment + "* Open Profit: "   + DoubleToStr(_orderInfo[4],2) + "\n";
    comment = comment + "* Open Profit +: " + DoubleToStr(_orderInfo[14],2) + "\n";
    comment = comment + "* Open Profit -: " + DoubleToStr(_orderInfo[15],2) + "\n";
    comment = comment + "* Open AccountProfit: " + DoubleToStr(AccountProfit(),2) + "\n";
    comment = comment + "-------------------------------------------------------------" + "\n";

    Comment(comment);
}

//+------------------------------------------------------------------+
//| Trailingstop logic. Applies a TS at profitable trades.           |
//+------------------------------------------------------------------+
int CheckTrailingStop()
{
   int ordercount =0;
   int ErrorNbr;
   // return only the market and pending orders.
   ordercount = OrdersTotal();
   int  n;
   for (int i = 0; i < ordercount; i++)
   {
      if(OrderSelect(i,SELECT_BY_POS, MODE_TRADES)==false) continue;

      // Only check current pair orders and orders which were created by this EA, by its magic number.
      if(OrderSymbol()!=_symbol) continue;
      if(OrderMagicNumber() != MagicNumber) continue;
            
      if(OrderType() != OP_BUY && OrderType() != OP_SELL) continue;
      
      //Normalize trailing stop value to the point value
      double TSTP = PipToPoints(TrailingSLValue) * Point; 
      if (OrderType() == OP_BUY)
      { 
            if ((Bid - OrderOpenPrice()) > TSTP)
            { 
               if (OrderStopLoss() < (Bid - TSTP))
               { 
                  OrderModify(OrderTicket(), OrderOpenPrice(), Bid - TSTP, OrderTakeProfit(), Gold); 
                  ErrorNbr=GetLastError();
                  if (ErrorNbr>1) Print(_FriendlyName + " Error#1080 - Error applying TrailingStop for Buy order: " + ErrorNbr);
               } 
            } 
      } // op_buy
      else if (OrderType() == OP_SELL)
      { 
         if ((OrderOpenPrice() - Ask) > TSTP)
         { 
            if ((OrderStopLoss() > (Ask + TSTP)) || (OrderStopLoss() == 0))
            { 
               OrderModify(OrderTicket(), OrderOpenPrice(), Ask + TSTP, OrderTakeProfit(), Gold); 
                  ErrorNbr=GetLastError();
                  if (ErrorNbr>1) Print(_FriendlyName + " Error#1081 - Error applying TrailingStop for Sell order: " + ErrorNbr);
            } 
         } 
      } // else (op_sell)
   } //for i

   return(0);
}

//+------------------------------------------------------------------+
//| This function retrieves information regarding the opened orders. |
//+------------------------------------------------------------------+
void CheckOrderInformation()
{
    // init array
    ArrayInitialize(_orderInfo,0);
    
    for(int i=0; i <= OrdersTotal(); i++)   {
        // Select only opened trades
        if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true)  {
            if(!OrderSymbol()==_symbol) continue;               // Filter pair
            if(OrderMagicNumber()==MagicNumber) {               // and number
                if(OrderType()==OP_BUY) {                       // group buy and short information seperatly
                    _orderInfo[0]=_orderInfo[0]+1;              // Totals
                    _orderInfo[2]=_orderInfo[2]+1;
                    _orderInfo[3]+=MarketInfo(Symbol(),MODE_BID)-OrderOpenPrice();      // Floating equity
                    _orderInfo[5]+=NormalizeDouble(OrderLots(),MarketInfo(Symbol(),MODE_DIGITS));   // total lots
                }
                if(OrderType()==OP_SELL)    {                   // see above
                    _orderInfo[1]=_orderInfo[1]+1;
                    _orderInfo[2]=_orderInfo[2]+1;
                    _orderInfo[3]+=OrderOpenPrice()-MarketInfo(Symbol(),MODE_ASK);
                    _orderInfo[6]+=NormalizeDouble(OrderLots(),MarketInfo(Symbol(),MODE_DIGITS));
                }
                _orderInfo[4]+=OrderProfit();                   // store profit of all combined orders
                _orderInfo[4]+=OrderSwap();
                _orderInfo[4]+=OrderCommission();
                if(OrderProfit()>=0)_orderInfo[14]=_orderInfo[14]+OrderProfit();
                if(OrderProfit()<=0)_orderInfo[15]=_orderInfo[15]+OrderProfit();
            }
        }
    }
    return;
}


//+------------------------------------------------------------------+
//| Returns the amount opened orders for this pair.                  |
//| if ordertype is -1 it will return all orders for this pair and   |
//| will not check for a specific ordertype.                         |
//+------------------------------------------------------------------+
int OrderCount(int orderType)
{
   int count =0,
       total;
   
    total=OrdersTotal();
    for (int intI = 0; intI < total; intI++)
    {  
        // Loop through opened orders
        OrderSelect(intI, SELECT_BY_POS, MODE_TRADES);    
        if ((OrderSymbol() == _symbol) && 
            (OrderMagicNumber()==MagicNumber)  && 
            ((OrderType()==orderType) || (orderType==-1))) count++ ;    // increase count
    }
   
   return (count);
}

//+-------------------------------------------------------------------+
//| This function opens an order and sets the initial tp and sl.      |
//+-------------------------------------------------------------------+
int OpenOrder(int orderType, double lotSize, string comment) 
{
    int ticket= 0,err = 0, attempt;    

    switch (orderType) {
    case 0:
        for (int i = 0; i < 100; i++) 
        {
            RefreshRates();
            if (true)
            {
                if(ContextIsFree()) ticket= OrderSend(Symbol(), OP_BUY, NormalizeDouble(lotSize,2), NormalizeDouble(MarketInfo(Symbol(),MODE_ASK),Digits), 3, 0, 0, comment, MagicNumber, 0, Green);
            }
            err = GetLastError();
            if (err == 0/* NO_ERROR */) break;
            if (err == 130){Print(_FriendlyName + "[1000] - #130: Open Buy Order Error takeprofit or stoploss to small.");break;}
            if (err == 4){Print(_FriendlyName + "[1001] - #004: Open Buy Order Error SERVER_BUSY.");break;}
            if (err == 137){Print(_FriendlyName + "[1002] - #137: Open Buy Order Error BROKER_BUSY.");break;}
            if (err == 146){Print(_FriendlyName + "[1003] - #146: Open Buy Order Error TRADE_CONTEXT_BUSY.");break;}
            if (err == 148){Print(_FriendlyName + "[1012] - #148: Open Buy Order Error ERR_TRADE_TOO_MANY_ORDERS.");break;}                  
            if (err == 136){Print(_FriendlyName + "[1004] - #136: Open Buy Order Error OFF_QUOTES.");break;}
            if (err > 0){Print(_FriendlyName + "[1005] - #" + err + ": Open Buy Order Error #",err);break;}
            Sleep(1000);
        }
        
        if(ticket>0) {
            if (OrderSelect(ticket,SELECT_BY_TICKET)==true) _lastOpenedTrade = OrderOpenTime();
        }
        
/*      
        if((ticket>0) && (!StopLoss==0))    {        
            attempt = 1;
            while (true) {
                OrderSelect(ticket,SELECT_BY_TICKET);  
                if(ContextIsFree()) OrderModify(ticket,OrderOpenPrice(),(OrderOpenPrice() - (PipToPoints(StopLoss)*Point)), 0,0,DodgerBlue);        // Set the stoploss for this new opened buy order        
                err=GetLastError();

                if (err == 0) break;
                attempt++;
                Sleep(250);
                if (attempt > 50) {
                   Print("HK_FourPeriods Error#1009-" + err + ": Could not set SL for new LONG: " + ticket + ". Please modify order " + ticket + " manually.");
                   break;
                }
             }    
        }      
      */
        break;
    case 1:
        for (i = 0; i < 100; i++) 
        {  
            RefreshRates();
            if (true)
            {
                if(ContextIsFree()) ticket= OrderSend(Symbol(), OP_SELL, NormalizeDouble(lotSize,2), NormalizeDouble(MarketInfo(Symbol(),MODE_BID),Digits), 3, 0, 0, comment, MagicNumber, 0, Red);
            }
            err = GetLastError();
            if (err == 0/* NO_ERROR */) break;
            if (err == 130){Print(_FriendlyName + "[1006] - #130: Open Sell Order Error takeprofit or stoploss to small.");break;}
            if (err == 4){Print(_FriendlyName + "[1007] - #004: Open Sell Order Error SERVER_BUSY.");break;}
            if (err == 137){Print(_FriendlyName + "[1008] - #137: Open Sell Order Error BROKER_BUSY.");break;}
            if (err == 146){Print(_FriendlyName + "[1009] - #146: Open Sell Order Error TRADE_CONTEXT_BUSY.");break;}
            if (err == 148){Print(_FriendlyName + "[1012] - #148: Open Sell Order Error ERR_TRADE_TOO_MANY_ORDERS.");break;}           
            if (err == 136){Print(_FriendlyName + "[1010] - #136: Open Sell Order Error OFF_QUOTES.");break;}
            if (err > 0){Print(_FriendlyName + "[1011] - #" + err +  ": Open Sell Order Error #",err);break;}
            Sleep(1000);
        }

        if(ticket>0) {
            if (OrderSelect(ticket,SELECT_BY_TICKET)==true) _lastOpenedTrade = OrderOpenTime();
        }
        
      /*
        if((ticket>0) && (!StopLoss==0))    {        
           attempt = 1;
           while (true) {
                OrderSelect(ticket,SELECT_BY_TICKET);    
                if(ContextIsFree()) OrderModify(ticket,OrderOpenPrice(),(OrderOpenPrice() + (PipToPoints(StopLoss)*Point)), 0,0,DodgerBlue);    // Set stoploss for this newly opened sell order
                err=GetLastError();

                if (err == 0) break;
                attempt++;
                Sleep(250);
                if (attempt > 50) {
                   Print("HK_FourPeriods Error#1007-" + err + ": Could not set SL for new SELL: " + ticket + ". Please modify order " + ticket + " manually.");
                   break;
                }
            }     
        }
        */
     break;
   }

    return (ticket);
}

//+--------------------------------------------------------------------------------------------+
//| Closes all open orders.                                                                    |
//| param: closingType. This indicates which orders needs to be closed.                        |      
//|                 -1: Close all orders, long and short due to a change in direction.         |                                                
//|                  0: Close all long orders.                                                 |                                                
//|                  1: Close all short orders.                                                |                                                
//+--------------------------------------------------------------------------------------------+
void CloseOrders(int closingType)
{
    int err, attempt;
    bool result;
    
    // Find all open orders
    for(int i = OrdersTotal() - 1; i >= 0; i--) {
        if (!OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) continue;
        if (OrderSymbol() != Symbol()) continue;
        if(!OrderMagicNumber()==MagicNumber) continue;
        
        // Look for closing all orders (LONG and SHORT)
        if(closingType==-1)  {
            attempt=1;
            while (true) {
              if (OrderType() == OP_BUY) // Close buy orders
                    if(ContextIsFree()) result = OrderClose(OrderTicket(), OrderLots(), Bid, 3,Blue);
              else if (OrderType() == OP_SELL) // Close sell order
                    if(ContextIsFree()) result = OrderClose(OrderTicket(), OrderLots(), Ask, 3,Blue);   
                err=GetLastError();
                if(err ==0 && (result)) break;
                attempt++;
                Sleep(250);
                if(attempt > 50)    {   // Retry 50 times, must be sufficient
                   Print(_FriendlyName + " Error#7010-" + err + ": Could not Close Order #" + OrderTicket() );
                   break;
                }
                RefreshRates(); // Update current quotes
            }
            
        }
        else  {
            // Look for closing all LONG orders
            if((closingType==0) && (OrderType()==OP_BUY)) { 
                attempt=1;
                while (true) {
                    if(ContextIsFree()) result = OrderClose(OrderTicket(), OrderLots(), Bid, 3,Blue);
                    err=GetLastError();
                    if(err ==0 && (result)) break;
                    attempt++;
                    Sleep(250);                    
                    if(attempt > 50)    {
                       Print(_FriendlyName + " Error#7011-" + err + ": Could not Close Long Order #" + OrderTicket() );
                       break;
                    }
                    RefreshRates();
                }
            }
            // Look for closing all SHORT orders
            if((closingType==1) && (OrderType()==OP_SELL)) { 
                attempt=1;
                while (true) {
                    if(ContextIsFree()) result = OrderClose(OrderTicket(), OrderLots(), Ask, 3,Blue);
                    err=GetLastError();
                    if(err ==0 && (result)) break;
                    attempt++;
                    Sleep(250);                    
                    if(attempt > 50)    {
                       Print(_FriendlyName + " Error#7012-" + err + ": Could not Close Short Order #" + OrderTicket() );
                       break;
                    }
                    RefreshRates();
                }
            }
        } 
    }           
    return(0); 
}


//+--------------------------------------------------------------------------------------------+
//| Checks to see whether the Trading Context is free. See err 146.                            |
//+--------------------------------------------------------------------------------------------+
bool ContextIsFree()
  {
    // check whether the trade context is free
    if(!IsTradeAllowed())
      {
        int StartWaitingTime = GetTickCount();
        Print("Trade context is busy. Wait for context to be free.");
        // infinite loop
        while(true)
          {
            // if the expert was terminated by the user, stop operation
            if(IsStopped()) 
              { 
                Print("The expert was stopped by the user."); 
                return(false);
               }
            // if it is waited longer than it is specified in the variable named 
            // MaxWaiting_sec, stop operation, as well
            if(GetTickCount() - StartWaitingTime > 30 * 1000) 
              {
                Print("The standby limit (" + 30 + " sec) has exceeded.");
                return(false);
              }
            // if the trade context has become free,
            if(IsTradeAllowed())
              {
                Print("Trade context is ready for transaction.");
                // refresh the market information
                RefreshRates();
                // leave the loop and start trading                
                break;
              }
            // Give some rest and try again.
            Sleep(100);
          }
      }
    else
        return(true);

    return(true);
  }
  
  
//+------------------------------------------------------------------+
//| Determines whether a new bar was openend or not.                 |
//+------------------------------------------------------------------+
bool NewBar() {
//	if (iTime(_symbol,_period,0) != _lastBarTime) {
//		_lastBarTime = iTime(_symbol,_period,0);		
	if (Time[0] != _lastBarTime) {
		_lastBarTime = Time[0];		
		
		return (true);
	} else
		return (false);
}

//+------------------------------------------------------------------+
//| Transforms any pip value to points/pip.                          |
//+------------------------------------------------------------------+
double PipToPoints(double pips)
  {
   double points;
   if(Close[0]<10) {
      points = pips*MathPow(10,Digits-4);
   }
   else if(Close[0]>10) {
      points = pips*MathPow(10,Digits-2);  
   }

   if(_digits == 3) points =  (pips*GetRealPoint(_digits)) / Point;    
   
   return(points);
  }

//+------------------------------------------------------------------+
//| Returns the used point value for the pair the ea is running on.  |
//+------------------------------------------------------------------+
double GetRealPoint(int digits) {
  switch (digits) {
    case 3: return(0.01);
    case 5: return(0.0001);
  }
  return(Point);
}

//+------------------------------------------------------------------+
//| Deletes all object which match a given prefix.                   |
//+------------------------------------------------------------------+
void ObDeleteObjectsByPrefix(string Prefix)
  {
   int L = StringLen(Prefix);
   int i = 0; 
   while(i < ObjectsTotal())
     {
       string ObjName = ObjectName(i);
       if(StringSubstr(ObjName, 0, L) != Prefix) 
         { 
           i++; 
           continue;
         }
       ObjectDelete(ObjName);
     }
  }